#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import zip, map, str, range
import ast
import re

import numpy as np
import pandas as pd
from sklearn.metrics import silhouette_score
from sklearn.cluster import KMeans


def create_compound_dict(converted_df):
    converted_df['Component Name'] = list(map(str.strip, converted_df['Component Name']))
    components = converted_df['Component Name'].unique().tolist()
    df = pd.DataFrame({'name': [],
                      'fragment': [],
                      'parent': [],
                      'child': []})
    for component in components:
        fragment = re.findall('/*\d+/\d+', component)[0]
        parent, child = fragment.split('/')
        for component_name in re.split('/*\d+/\d+', component):
            if component_name !='':
                name = component_name
        df = df.append(pd.Series([name, fragment, parent, child], index=['name','fragment', 'parent', 'child']), ignore_index=True)
    compound_list = df.name.unique().tolist()
    compound_dict = {}
    for compound in compound_list:
        ndf = df.loc[df['name'] == compound].reset_index(drop=True)
        pairs = []
        for index, rows in ndf.iterrows():
            pair = [rows['parent'], rows['child']]
            pairs.append(pair)
            compound_dict.update({compound : pairs})
    return compound_dict


def get_clusters(compound):
    """
    This function check silhouette score for possibility of 2 clusters and 
    based on that it returns number of cluster
    """
    X = np.array(compound)
    try:
        for n_cluster in range(2, 11):
            kmeans = KMeans(n_clusters=n_cluster).fit(X)
            label = kmeans.labels_
            sil_coeff = silhouette_score(X, label, metric='euclidean')
            k = 1
            if sil_coeff > 0.85:
                k = n_cluster
        return k
    except:
        return 1


def cluster_no(compound_dict):
    """
    This function create dictionary for all 
    metabolites along with number of possible clusters

    """
    cluster_dict = {}
    for key, value in list(compound_dict.items()):
        k = get_clusters(value)
        cluster_dict.update({key: k})
    return cluster_dict

        
def seperate_fragments(fragments,cluster_label):
    """This function seperate components based on cluster"""
    parent_1 = []
    parent_2 = []
    for f,l in zip(fragments,cluster_label):
        if l:
            parent_1.append(f)
        else:
            parent_2.append(f)
    return parent_1, parent_2


def create_cluster_new_dict(raw_df):
    """
    This function creates clusters for fragments in the 
    the raw dtaframe and formats them into a new dictionary
    Args
    -----
        raw_df: dataframe
        raw intensity dataframe of file
    returns
    -------
        new_dict: dictionary object
        new dictionary of clusters
    """

    compound_dict = create_compound_dict(raw_df)
    cluster_dict = cluster_no(compound_dict)
    final_dict = {}
    new_dict = {}
    for key, value in list(cluster_dict.items()):
        fragment_dict = {}
        if value ==2:
            kmeans = KMeans(n_clusters=value).fit(compound_dict[key])
            label = list(kmeans.labels_)
            parent1, parent2 = seperate_fragments(compound_dict[key], label)
            fragment_dict = { key + '_1': parent1, key + '_2': parent2}
            for key, value in list(fragment_dict.items()):
                c = sorted(fragment_dict[key],key=lambda x: x[0])
                new_dict.update({key +  '+' + str(c[0]): fragment_dict[key]})
        else:
            c = sorted(compound_dict[key],key=lambda x: x[0])
            new_dict.update({key + '+'+ str(c[0]): compound_dict[key]})
    return new_dict

def convert_in_string(a):
    """
    converts a list into string joining by '/'
    """
    my_lst_str = '/'.join(map(str, a))
    return my_lst_str


def convert_string_to_required_String(a):
    """
    converts the string into the required format
    """
    my_lst_str = '/'.join(map(str, ast.literal_eval(a)))
    return my_lst_str


def correct_metabolite_name(name):
    """
    Creates the metabolite name in the required format

    Parameters
    ----------
        name: name of the metabolite

    Returns
    -------
        corrected_name: name in the required format with "_"
    """
    corrected_name = name.split('_')[0]
    return corrected_name


def create_metadata_mq(raw_df):
    """
    This function if the central function in creating metadata 
    file from the intensity file and returns the dataframe of
    the mapped unlabeled fragments with their daughter fragments
    Parameters:
    -----------
        raw_df: Dataframe
        dataframe of raw intensity file
    Returns:
    ---------
        genrated new metadata dataframe.
    """
    
    new_dict = create_cluster_new_dict(raw_df)
    metadata_mq = pd.DataFrame()
    for key, value in list(new_dict.items()):
        for i in range(len(value)):
            metadata_mq  = metadata_mq.append(pd.Series([key, value[i]], index=['Unlabeled fragment', 'Component Name']), ignore_index=True)

    metadata_mq['Component Name'] = metadata_mq['Component Name'].apply(convert_in_string)
    metadata_mq['Unlabeled fragment'] = list(map(str.strip, metadata_mq['Unlabeled fragment']))
    udf = metadata_mq['Unlabeled fragment'].str.split('+', 1, expand=True)
    udf[1] = udf[1].apply(convert_string_to_required_String)
    udf[0] = udf[0].apply(correct_metabolite_name)
    udf[2] = metadata_mq['Component Name']
    final_df = pd.DataFrame()
    final_df["Component Name"] = udf[0].map(str) + udf[2]
    final_df["Unlabeled Fragment"] = udf[0].map(str) + udf[1]
    final_df = final_df.replace({'M-1 D4-Taurine128/80': '128/80 M-1 D4-Taurine',
                                 'M-1 Taurine124/80': '124/80 M-1 Taurine'})
    
    return final_df